/*
 * sxid - suid, sgid file and directory checking
 * conffile.c - configuration file functions
 *
 * Copyright (C) 1998 Ben Collins <bcollins@debian.org>
 *
 * This is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2,
 * or (at your option) any later version.
 *
 * This is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with sxid; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <ctype.h>
#include <strings.h>

#include "config.h"
#include "structs.h"
#include "globals.h"
#include "conffile.h"

struct config_opt config_options;
static char conf_default[] = CONF_FILE;
char *confname = conf_default;

int get_value (char *key, char *value)
{
    FILE *conffile;
    char buffer[1024];
    int count = 0;
    int line_no = 0;

    bzero (buffer, sizeof (buffer));

    if ((conffile = fopen (confname, "r")) == NULL) {
	return 1;
    } else {
	while ((fgets (buffer, sizeof (buffer), conffile)) != NULL) {
	    if (buffer[0] != '#') {
		if (strstr (buffer, key) != (char) NULL) {
		    if (strstr (buffer, "\"") == (char) NULL) {
			fclose (conffile);
			return line_no;
		    }
		    strncpy (buffer, strstr (buffer, "\"") + 1, MAXBUF);
		    buffer[MAXBUF - 1] = '\0';

		    count = 0;
		    while (count < sizeof (buffer)) {
			if ((isprint (buffer[count])) && buffer[count] != '"')
			    value[count] = buffer[count];
			else {
			    value[count] = '\0';
			    break;
			}
			count++;
		    }
		    value[sizeof (buffer) - 1] = '\0';
		    fclose (conffile);
		    return 0;
		}
	    }
	    line_no++;
	}
	fclose (conffile);
	return -1;
    }

}

void init_conf (void)
{
    char temp[1024];

    check_return (get_value ("SEARCH", config_options.search_paths), "SEARCH");
    check_return (get_value ("EXCLUDE", config_options.exclude_paths), "EXCLUDE");
    check_return (get_value ("EMAIL", config_options.email), "EMAIL");
    check_return (get_value ("LOG_FILE", config_options.log_file), "LOG_FILE");
    check_return (get_value ("FORBIDDEN", config_options.forbidden_paths), "FORBIDDEN");

    check_return (get_value ("KEEP_LOGS", temp), "KEEP_LOGS");
    config_options.keep_logs = atoi (temp);

    check_return (get_value ("ALWAYS_NOTIFY", temp), "ALWAYS_NOTIFY");
    if (isyes (temp))
	config_options.flags |= FLAG_ANOT;

    check_return (get_value ("ALWAYS_ROTATE", temp), "ALWAYS_ROTATE");
    if (isyes (temp))
	config_options.flags |= FLAG_AROT;

    check_return (get_value ("ENFORCE", temp), "ENFORCE");
    if (isyes (temp))
	config_options.flags |= FLAG_EFOR;

    check_return (get_value ("LISTALL", temp), "LISTALL");
    if (isyes (temp))
	config_options.flags |= FLAG_LALL;

    return;
}

void check_return (int line, char *key)
{
    if (!line)
	return;
    switch (line) {
    case -1:
	fprintf (stderr, "Option \'%s\' not found in \'%s\'.\n", key, confname);
	break;

    default:
	fprintf (stderr, "Error in \'%s\' line %d.\n", confname, line);
    }
    exit (1);
}
